'use strict';
/**
 * Start the requirejs config.
 */
require.config({
  paths: {
    'bootstrap-switch': '../../webjars/bootstrap-switch/3.3.2/js/bootstrap-switch.min'
  },
  shim: {
    'bootstrap-switch' : ['jquery']
  }
});
/**
 * Start the real script.
 */
require(['jquery', 'bootstrap', 'bootstrap-switch', 'xwiki-events-bridge'], function ($) {

  /**
   * Page initialization
   */
  $(document).ready(function() {
    var initBootstrapSwitches = function () {
      $('#notificationSystemFilterPreferencesLiveTable .notificationFilterPreferenceCheckbox, #notificationSystemFilterPreferencesLiveTable .toggleableFilterPreferenceCheckbox').bootstrapSwitch({
        size: 'mini',
        labelText: '<img src=\"..\/..\/resources\/icons\/silk\/bell.png\" alt=\"Icon\" \/>'
      });
    };
    $(document).on('xwiki:livetable:displayComplete', function (event, data) {
      // Enable bootstrap switches
      initBootstrapSwitches();
    });
    initBootstrapSwitches();

  });
});

define('SystemToggleableFilterPreference', ['jquery', 'bootstrap-switch'], function($) {
  /**
  * Construct the Toggleable Filter Preference object.
  */
  return function(domElement, docURL, row) {
    var self               = this;
    self.domElement        = $(domElement);
    self.docURL           = docURL;
    self.row               = row;
    /**
     * Initialization
     */
    self.init = function () {
      // On change
      self.domElement.off('switchChange.bootstrapSwitch.systemFilter')
        .on('switchChange.bootstrapSwitch.systemFilter', function(event, state) {
          setTimeout(function() { self.save(state); }, 1);
        });
    };

    /**
     * Save the filter preference
     */
    self.save = function(state) {
      var notification = new XWiki.widgets.Notification("\u0412\u0430\u0448\u0438 \u043D\u0430\u0441\u0442\u0440\u043E\u0439\u043A\u0438 \u0441\u043E\u0445\u0440\u0430\u043D\u044F\u044E\u0442\u0441\u044F ...", 'inprogress');
      var filterName = self.domElement.attr('data-filterName');
      var objectNumber = self.domElement.attr('data-objectNumber');
      if (objectNumber == '') {
        // If the object does not exist yet, create it (make this change a minor edit so it's not displayed, by default, in notifications)
        var restURL = self.docURL + '/objects?media=json&minorRevision=true';
        var params = {
          'className': 'XWiki.Notifications.Code.ToggleableFilterPreferenceClass',
          'property#filterName': filterName,
          'property#isEnabled': state ? '1' : '0'
        };
        $.post(restURL, params).done(function (data) {
          self.domElement.attr('data-objectNumber', data.number);
          notification.hide();
          new XWiki.widgets.Notification("\u041D\u0430\u0441\u0442\u0440\u043E\u0439\u043A\u0438 \u0444\u0438\u043B\u044C\u0442\u0440\u0430 \u0441\u043E\u0445\u0440\u0430\u043D\u0435\u043D\u044B!", 'done');
        }).fail(function () {
          notification.hide();
          new XWiki.widgets.Notification("\u041D\u0435 \u0443\u0434\u0430\u043B\u043E\u0441\u044C \u0441\u043E\u0445\u0440\u0430\u043D\u0438\u0442\u044C \u043D\u0430\u0441\u0442\u0440\u043E\u0439\u043A\u0438 \u0444\u0438\u043B\u044C\u0442\u0440\u0430", 'error');
        });
      } else if (objectNumber != '') {
        // If the object already exist, just update its value (make this change a minor edit so it's not displayed, by default, in notifications)
        var restURL = self.docURL + '/objects/XWiki.Notifications.Code.ToggleableFilterPreferenceClass/' + objectNumber
          + '/properties/isEnabled?media=json&minorRevision=true';
        $.ajax(restURL, {
          method: 'PUT',
          contentType: 'text/plain',
          data: state ? '1' : '0'
        }).done(function (data) {
          notification.hide();
          new XWiki.widgets.Notification("\u041D\u0430\u0441\u0442\u0440\u043E\u0439\u043A\u0438 \u0444\u0438\u043B\u044C\u0442\u0440\u0430 \u0441\u043E\u0445\u0440\u0430\u043D\u0435\u043D\u044B!", 'done');
          $(document).trigger('xwiki:livetable:notifications-filters-system:toggle', {'row': self.row, 'state': state});
        }).fail(function () {
          notification.hide();
          new XWiki.widgets.Notification("\u041D\u0435 \u0443\u0434\u0430\u043B\u043E\u0441\u044C \u0441\u043E\u0445\u0440\u0430\u043D\u0438\u0442\u044C \u043D\u0430\u0441\u0442\u0440\u043E\u0439\u043A\u0438 \u0444\u0438\u043B\u044C\u0442\u0440\u0430", 'error');
        });
      }
    };

    // Call init
    self.init();
  };
});

require([
  'jquery',
  'SystemToggleableFilterPreference',
  'xwiki-meta',
  'bootstrap-switch',
  'xwiki-events-bridge'
], function ($, SystemToggleableFilterPreference, xm) {
  $(document).on('xwiki:livetable:notifications-filters-system:toggle', function(event, data) {
    var changedRow = $(data.row.isEnabled);
    if (changedRow.length > 0) {
      if (data.state) {
        changedRow.attr("checked", "checked");
      } else {
        changedRow.removeAttr("checked");
      }
      data.row.isEnabled = changedRow.get(0).outerHTML;
    }
  });

  // Globals
  var serviceReference = XWiki.Model.resolve('XWiki.Notifications.Code.NotificationPreferenceService',
    XWiki.EntityType.DOCUMENT);
  var serviceURL = new XWiki.Document(serviceReference).getURL('get', 'outputSyntax=plain');
  var target = $('.systemFilterPreferences').attr('data-target');

  var docURL = $('.systemFilterPreferences').attr('data-doc-url');
  var postParams = {
    action: 'setFilterPreferenceEnabled',
    csrf: xm.form_token,
    target: target
  };
  if (target == 'user') {
    var targetUser = $('.systemFilterPreferences').attr('data-user');
    postParams['user'] = targetUser;
  };

  var enhanceRow = function(row, data) {
    // Enable / Disable action for classic filter preferences
    $(row).find('.notificationFilterPreferenceCheckbox').off('switchChange.bootstrapSwitch.systemFilter')
    .on('switchChange.bootstrapSwitch.systemFilter', function(event, state) {
      var notif = new XWiki.widgets.Notification("Сохранение...", 'inprogress');
      var params = {
        filterPreferenceId: data.filterPreferenceId,
        enabled: state
      };
      $.post(serviceURL, $.extend({}, params, postParams)).done(function() {
        notif.replace(new XWiki.widgets.Notification("Настройки фильтра сохранены!", 'done'));
        $(document).trigger('xwiki:livetable:notifications-filters-system:toggle', {'row': data, 'state': state});
      }).fail(function() {
        notif.replace(new XWiki.widgets.Notification("Не удалось сохранить настройки фильтра", 'error'));
      });
    });
    // Enable / Disable action for toggleable filter preferences
    $(row).find('.toggleableFilterPreferenceCheckbox').each(function() {
      new SystemToggleableFilterPreference(this, docURL, data);
    });
  };

  // Callback on livetable row printing
  $(document).on('xwiki:livetable:notificationSystemFilterPreferencesLiveTable:newrow', function(event, data) {
    enhanceRow(data.row, data.data);
  });

  // Initializer
  $(function() {
    // This script is usually loaded after the live table is displayed and so the previous callback might not be called.
    $('#notificationSystemFilterPreferencesLiveTable-display > tr[data-index]').each(function() {
      var data = window.livetable_notificationSystemFilterPreferencesLiveTable.fetchedRows[$(this).data('index')];
      enhanceRow(this, data);
    });
  });
});
